@@ -12,7 +12,7 @@
 ##############################################################################
 """ Basic ZPublisher request management.
 
-$Id: BaseRequest.py 104361 2009-09-18 20:26:31Z mj $
+$Id: BaseRequest.py 105078 2009-10-15 11:03:28Z witsch $
 """
 from urllib import quote as urllib_quote
 import xmlrpc
@@ -211,8 +211,10 @@ class BaseRequest:
         self._held=None
 
     def close(self):
-        self.clear()
         notify(EndRequestEvent(None, self))
+        # subscribers might need the zodb, so `clear` must come afterwards
+        # (since `self._held=None` might close the connection, see above)
+        self.clear()
 
     def processInputs(self):
         """Do any input processing that could raise errors
